from django.db.models import Q
from django.db.models.sql.constants import GET_ITERATOR_CHUNK_SIZE, NO_RESULTS
from django.db.models.sql.where import WhereNode

from .utils import get_current_tenant, get_tenant_filters


def add_tenant_filters_on_query(obj):
    current_tenant = get_current_tenant()

    if current_tenant:
        try:
            filters = get_tenant_filters(obj.model)
            obj.add_q(Q(**filters))
        except ValueError:
            pass


def wrap_get_compiler(base_get_compiler):
    # Adds tenant filters to the query object
    def get_compiler(obj, *args, **kwargs):
        add_tenant_filters_on_query(obj)
        return base_get_compiler(obj, *args, **kwargs)

    get_compiler._sign = "get_compiler django-multitenant"
    return get_compiler


def wrap_update_batch(base_update_batch):
    # Written to decorate the update_batch method of the UpdateQuery class to add tenant_id filters.
    # Since add tenant_filters method must be executed before the execute_sql method, we have to
    # copy and rewrite the update_batch method.
    # CAUTION: Since source is copied, UpdateQuery.update_batch method should be tracked
    def update_batch(obj, pk_list, values, using):
        obj.add_update_values(values)
        for offset in range(0, len(pk_list), GET_ITERATOR_CHUNK_SIZE):
            obj.where = WhereNode()
            obj.add_q(Q(pk__in=pk_list[offset : offset + GET_ITERATOR_CHUNK_SIZE]))
            add_tenant_filters_on_query(obj)
            obj.get_compiler(using).execute_sql(NO_RESULTS)

    update_batch._sign = "update_batch django-multitenant"
    return update_batch



from os import path

from setuptools import find_packages, setup

# read the contents of your README file
this_directory = path.dirname(path.abspath(__file__))

with open(path.join(this_directory, "README.md")) as f:
    long_description = f.read()


# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    name="django-islands",
    version="0.0.3",  # Required
    description="Django Library to Implement Multi-Tenant Architecture via tenant_id column in Postgres",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yellow-devs/django-islands",
    author="Davi Silva Rafacho",
    author_email="davi.s.rafacho.developer@gmail.com",
    # Classifiers help users find your project by categorizing it.
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Database",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Framework :: Django :: 5",
        "Framework :: Django :: 5.0",
        "Framework :: Django :: 5.1",
        "Framework :: Django :: 5.2",
    ],
    keywords=("django multi-tenant postgres saas"),
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests", "docs", "docs.*"]),
)
